'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

function create(fn) {
  var state = {
    listeners: [],
    current: fn(function (merge) {
      if (typeof merge === 'function') merge = merge(state.current);
      state.current = Object.assign({}, state.current, merge);
      state.listeners.forEach(function (listener) {
        return listener(state.current);
      });
    }, function () {
      return state.current;
    })
  };
  return [// useStore
  function (selector, dependencies) {
    var selected = selector ? selector(state.current) : state.current; // Using functional initial b/c selected itself could be a function

    var _React$useState = React.useState(function () {
      return selected;
    }),
        slice = _React$useState[0],
        set = _React$useState[1];

    var sliceRef = React.useRef();
    React.useEffect(function () {
      return void (sliceRef.current = slice);
    }, [slice]);
    React.useEffect(function () {
      var ping = function ping() {
        // Get fresh selected state
        var selected = selector ? selector(state.current) : state.current; // If state is not equal from the get go and not an atomic then shallow equal it

        if (sliceRef.current !== selected && typeof selected === 'object' && !Array.isArray(selected)) {
          selected = Object.entries(selected).reduce(function (acc, _ref) {
            var _Object$assign;

            var key = _ref[0],
                value = _ref[1];
            return sliceRef.current[key] !== value ? Object.assign({}, acc, (_Object$assign = {}, _Object$assign[key] = value, _Object$assign)) : acc;
          }, sliceRef.current);
        } // Refresh local slice, functional initial b/c selected itself could be a function


        if (sliceRef.current !== selected) set(function () {
          return selected;
        });
      };

      state.listeners.push(ping);
      return function () {
        return state.listeners = state.listeners.filter(function (i) {
          return i !== ping;
        });
      };
    }, dependencies || [selector]); // Returning the selected state slice

    return selected;
  }, {
    subscribe: function subscribe(fn) {
      state.listeners.push(fn);
      return function () {
        return state.listeners = state.listeners.filter(function (i) {
          return i !== fn;
        });
      };
    },
    getState: function getState() {
      return state.current;
    },
    destroy: function destroy() {
      return state.listeners = [], state.current = {};
    }
  }];
}

module.exports = create;
